import sys
import re

class statement:
    def __init__(self, num):
        self.location = ''
        self.result = ''
        self.operand = ''
        self.comments = ''
        self.valid_instr = 0

class instruction:
    def __init__(self, num):
        self.code = ""          #16 bits
        self.gh = ""            #7 bits
        self.g = ""              #4 bits
        self.h = ""              #3 bits 
        self.i = ""              #3 bits
        self.j = ""              #3 bits
        self.k = ""              #3 bits
        self.jk = ""             #6 bits
        self.jkm = ""            #22 bits
        self.ijkm = ""           #25 bits
        self.type = ""
        self.instruction = ""

def expr_eval(expression):
    #first split the expression into individual tokens to parse
    tokens = re.compile('(\+|\-|\*|\/)')    #putting the regex in ()'s includes the tokens for some reason
    tokens = tokens.split(expression)
    if "" in tokens:
        tokens.remove("")
    result = 0
    i=0
    mode = "+"
    #now let's evaluate the expression
    while(i<len(tokens)):
        #First, check if it's a number or an operator
        if tokens[i] != "+" and tokens[i] != "-" and tokens[i] != "*" and tokens[i] != "/":
            #It's a number! Now, let's convert it into an integer
            if tokens[i] in a_type or tokens[i] in s_type or tokens[i] in v_type:
                print "Error! Trying to evaluate regs as expression"
                break
            elif tokens[i] in symbols:                            #it's an already-defined symbol!
                tokens[i]=symbols[tokens[i]]
            elif tokens[i][0]=="O" and tokens[i][1]=="'":       #it's an octal number
                tokens[i]=int("0" + tokens[i].lstrip("O'"),8)   #chop off the leading 2 characters and replace with a "0" to make it python-friendly
                #print "explicit octal detected"
            elif tokens[i][0]=="D" and tokens[i][1]=="'":       #it's a decimal number
                tokens[i]=int(tokens[i].lstrip("D'"))           #chop off the leading 2 characters to make it python friendly
            elif tokens[i][0]=="A" and tokens[i][1]=="'" or tokens[i][0]=="'":  #it's an ASCII string
                tokens[i]=tokens[i]
            else:                                               #It's an octal number, but with no "O'" prefix
                tokens[i]=int("0" + tokens[i],8)
                #print "implicit octal detected"
            
            if mode == "+":
                result = result + tokens[i]
            elif mode == "-":
                result = result - tokens[i]
            elif mode == "*":
                result = result*tokens[i] 
            elif mode == "/":
                result = result/tokens[i]
        else:
            mode = tokens[i]
        i=i+1
            
    return result

mode = "normal"
input_file = sys.argv[1]
if len(sys.argv)==3:
    mode = sys.argv[2]
    print "mode:" + mode

infile = open(input_file,"r")
raw_assembly = infile.readlines()

formatted_code = []
symbols = {}
s_type = ["S0","S1","S2","S3","S4","S5","S6","S7","S."]
a_type = ["A0","A1","A2","A3","A4","A5","A6","A7","A."]
v_type = ["V0","V1","V2","V3","V4","V5","V6","V7","V."]

#First, we want to load in the raw text file and
#parse it into "Location", "Result", "Operand" and "Comments" fields
for line in raw_assembly:
    index = len(formatted_code)
    split_line = line.split()
    formatted_code.append(statement(index))
    #First, check if it is a comment-only line
    if line[0]=="*":
        comment = ""
        for i in split_line:
            comment = comment + i + " "
        formatted_code[index].comments = comment
    elif len(line) > 35 and line[0:34]=="                                   ":
        comment = ""
        for i in split_line:
            comment = comment + i + " "
        formatted_code[index].comments = comment
    #Then, check if the Location field exists or not
    elif line[0]==" " and line[1]==" ":
        formatted_code[index].result = split_line[0]
        formatted_code[index].operand = split_line[1]
        comment = ""
        for i in split_line[2:]:
            comment = comment + i + " "
        formatted_code[index].comments = comment
        if formatted_code[index].result != "=":         #detect symbols
            formatted_code[index].valid_instr = 1
    #Otherwise, it's just a normal instruction
    else:
        formatted_code[index].location = split_line[0]
        formatted_code[index].result = split_line[1]
        formatted_code[index].operand = split_line[2]
        comment = ""
        for i in split_line[3:]:
            comment = comment + i + " " 
        formatted_code[index].comments = comment
        if formatted_code[index].result != "=":         #detect symbols
            formatted_code[index].valid_instr = 1

#We now have all of our code parsed into fields and stored in the "formatted code" array.
#Let's start turning it into instructions!

if mode=="debug":
    for i in formatted_code:
        print "location: " + i.location
        print "result: " + i.result
        print "operand: " + i.operand
        print "comments: " + i.comments
        print "valid instr: " + str(i.valid_instr)
        print ""

for i in formatted_code:
    if i.result=="=":
        if mode=="debug":
            print "Symbol found!"
        symbols[i.location]=expr_eval(i.operand)

if mode =="debug":
    print "Symbols:"
    print symbols

#Let's go through and start defining instructions
instr_code = []
for i in formatted_code:
    index = len(instr_code)
    tmp_code = ""
    if i.valid_instr==1:
        if i.result == "VL":
        #002 VL     AK      Transmit Ak to VL
            instr_code.append(instruction(index))
            instr_code[index].instruction = i
            if i.operand[0]!="A":
                print "Error! Can't assign non-A# reg to VL"
                break
            tmp_code = "00200" + i.operand[1]
            instr_code[index].code = tmp_code;

        elif i.result == "VM":                 
        #003 VM     Sj      Transmit Sj to VM
            instr_code.append(instruction(index))
            instr_code[index].instruction = i
            if i.operand[0]!="S":
                print "Error! Can't assign non-S# reg to VM"
                break
            tmp_code = "0030" 
            if i.operand[1] != ".":
                tmp_code = tmp_code + i.operand[1]
            else:
                tmp_code = tmp_code + symbols[i.operand[2:]]
            tmp_code = tmp_code + "0"
            instr_code[index].code = tmp_code

        elif i.result == "EX":
        #004 EX     Oprnd   Normal Exit
            print "Not currently support!"
            break
        
        elif i.result == "J" and i.operand[0]=="B":
        #005 J      Bjk     Branch to (Bjk)
        #This instruction sets the P register to the parcel address specified
        #by the contents of Bjk, and execution continues at that address.
            instr_code.append(instruction(index))
            instr_code[index].instruction = i
            tmp_code = "0050"
            if i.operand[1] != ".":     #it's an explicit register
                if len(i.operand)==3:
                    tmp_code = tmp_code + i.operand[1] + i.operand[2]
                elif len(i.operand)==2:
                    tmp_code = tmp_code + "0" + i.operand[1]
                else:
                    print "Error! Format must be B# or B##"
                    break
            else:                    #it's a symbol for the register value
                if len(oct(symbols[i.operand[2:]]))==2:
                    tmp_val = oct(symbols[i.operand[2:]])
                    tmp_code = tmp_code + "0" + tmp_val[1]
                else:
                    tmp_val = oct(symbols[i.operand[2:0]])
                    tmp_code = tmp_code + tmp_val[1] + tmp_val[2]
                
            instr_code[index].code = tmp_code

        elif i.result == "J" and i.operand[0] != "B":           
        #006 J      exp     Branch to ijkm
        #This instruction sets the P register to the parcel address specified
        #by the low order 24 bits of the ijkm field, and execution continues at
        #that address. The high order bit of the ijkm field is ignored.
            instr_code.append(instruction(index))               #add the op-code + ijk
            instr_code.append(instruction(index+1))             #add "m"
            instr_code[index].instruction = i
            tmp_code = "006"
            tmp_val = "%06x" % expr_eval(i.operand)     #tmp_val now holds the destination as a 6-digit hex string value
            mval    = "%06o" % int(tmp_val[2:6],16)     #figure out the lowest 16 bits of the destination (the "M" value), and convert to 6-digit octal
            instr_code[index+1].code = mval             #write out the 6-digit octal value as the following 16-bit instruction
            tmp_code = tmp_code + "%03o" % int(tmp_val[0:2],16) #now write out the remaining part of the address as the lower 9 bits of the jump-instruction
            instr_code[index].code = tmp_code

        elif i.result == "R":
        #007 R     exp     Return jump to ijkm
        #This instruction sets register B00 to the address of the following
        #parcel. The P register is then set to the parcel address specified
        #by the low order 24 bits of the ijkm field, and execution continues 
        #at that address. The high order bit of the ijkm field is ignored.
        #The purpose of this instruction is to provie a return linkage for
        #subroutine calls. The subroutine is entered via a return jump. The
        #subroutine returns to the caller at the instruction following the call
        #by executing a branch instruction on the contents of B00 (005)
            instr_code.append(instruction(index))               #add the op-code + ijk
            instr_code.append(instruction(index+1))             #add "m"
            instr_code[index].instruction = i
            tmp_code = "007"
            tmp_val = "%06x" % expr_eval(i.operand)     #tmp_val now holds the destination as a 6-digit hex string value
            mval    = "%06o" % int(tmp_val[2:6],16)     #figure out the lowest 16 bits of the destination (the "M" value), and convert to 6-digit octal
            instr_code[index+1].code = mval             #write out the 6-digit octal value as the following 16-bit instruction
            tmp_code = tmp_code + "%03o" % int(tmp_val[0:2],16) #now write out the remaining part of the address as the lower 9 bits of the jump-instruction
            instr_code[index].code = tmp_code

        elif i.result in ["JAZ","JAN","JAP","JAM","JSZ","JSN","JSP","JSM"]:
        #These instructions test the contents of A0 for the condition speci-
        #fied. If the condition is satisfied, the P register is set to the
        #parcel address specified by the low order 24 bits of the ijkm field,
        #and execution continues at that address. The high order bit of the 
        #ijkm field is ignored. If the condition is not satisfied, execution
        #continues with the instruction following the branch instruction. If
        #A0 contains zero, it is considered positive.
            instr_code.append(instruction(index))               #add the op-code + ijk
            instr_code.append(instruction(index+1))             #add "m"
            instr_code[index].instruction = i
            if i.result=="JAZ":
                tmp_code = "010"
            elif i.result=="JAN":
                tmp_code = "011"
            elif i.result=="JAP":
                tmp_code = "012"
            elif i.result=="JAM":
                tmp_code = "013"
            elif i.result=="JSZ":
                tmp_code = "014"
            elif i.result=="JSN":
                tmp_code = "015"
            elif i.result=="JSP":
                tmp_code = "016"
            else:       #JSM
                tmp_code = "017"
            tmp_val = "%06x" % expr_eval(i.operand)     #tmp_val now holds the destination as a 6-digit hex string value
            mval    = "%06o" % int(tmp_val[2:6],16)     #figure out the lowest 16 bits of the destination (the "M" value), and convert to 6-digit octal
            instr_code[index+1].code = mval             #write out the 6-digit octal value as the following 16-bit instruction
            tmp_code = tmp_code + "%03o" % int(tmp_val[0:2],16) #now write out the remaining part of the address as the lower 9 bits of the jump-instruction
            instr_code[index].code = tmp_code

        elif i.result[0:2] in a_type and i.operand[0:2] not in a_type and i.operand[0:2] not in s_type:
        #020 Ai         exp     Transmit jkm to Ai
        #    Ai         #exp    
        #This two-parcel instruction enters the 22-bit quantity of the jkm
        #field into Ai. The quantity is treated as a 22-bit positive integer;
        #the upper bits of Ai are cleared. The assembler generates this in-
        #struction when no #symbol precedes the expression and the value of
        #the expression exceeds 63. If all symbols in the expression have not
        #been previously defined, this instruction is generated when the ex-
        #pression value is positive even though the value may be less than
        #64. When the # symbol precedes the expression, the expression is
        #first evaluated and if the value is negative, it is complemented and
        #the complemented value is stored in the jkm field. The complement is 
        #formed by changing all 1 bits to zero and all 0 bits to one. When the
        #expression is positive, an 021 instruction is generated (see below).
            instr_code.append(instruction(index))               #add the op-code + ijk
            instr_code.append(instruction(index+1))             #add "m"
            instr_code[index].instruction = i
            tmp_code = "020"
            if i.result[1]!=".":
                tmp_code = tmp_code + i.result[1]
            else:
                tmp_code = tmp_code + str(symbols[i.result[1]])
            
            tmp_val = "%06x" % expr_eval(i.operand)     #tmp_val now holds the destination as a 6-digit hex string value
            mval    = "%06o" % int(tmp_val[2:6],16)     #figure out the lowest 16 bits of the destination (the "M" value), and convert to 6-digit octal
            instr_code[index+1].code = mval             #write out the 6-digit octal value as the following 16-bit instruction
            tmp_code = tmp_code + "%02o" % (int((tmp_val[0:2]),16) & 0x3f)  #now write out the remaining part of the address as the lower 6 bits of the jump-instruction
            instr_code[index].code = tmp_code




for instr in instr_code:
    if(instr.instruction==""):
        print instr.code
    elif (instr.instruction.comments!=""):
        print instr.code + "  " + instr.instruction.comments
    else:
        print instr.code








